/*
 * game.hpp - XD-Ball
 * Copyright (c) 2011, 2012 - TÖRÖK Attila (torokati44@gmail.com)
 *
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 *   1. The origin of this software must not be misrepresented; you must not
 *      claim that you wrote the original software. If you use this software
 *      in a product, an acknowledgment in the product documentation would be
 *      appreciated but is not required.
 *
 *	 2. Altered source versions must be plainly marked as such, and must not be
 *      misrepresented as being the original software.
 *
 *	 3. This notice may not be removed or altered from any source
 *      distribution.
 */

#ifndef GAME_HPP_INCLUDED
#define GAME_HPP_INCLUDED

#include <vector>

#include <SFML/System/Vector2.hpp>
#include <SFML/Window/Event.hpp>
#include <SFML/Audio/Sound.hpp>
#include <SFML/Audio/Music.hpp>

#include "statebase.hpp"
#include "ball.hpp"
#include "brick.hpp"
#include "paddle.hpp"
#include "particlesystem.hpp"
#include "background.hpp"
#include "guielementcontainer.hpp"
#include "guistring.hpp"

class Game: public StateBase
{
private:
	float time;
	
	unsigned int score;
	unsigned int combo;
	
	char buffer[64];
	
	std::vector< Ball* > balls;
	std::vector< Brick* > bricks;
	Paddle* paddle;
	ParticleSystem particle_system;
	Background background;
	
	GUIElementContainer gui;
	GUIString* combo_string;
	GUIString* score_string;
	
	std::vector< sf::Sound* > sounds;
	sf::Music* music;
	
	void play_sound(const std::string&, float = 400.0f);
	
public:
	Game(unsigned int, unsigned int);
	
	virtual void process_events(std::vector< sf::Event >&);
	virtual void render();
	virtual void update(float);
	virtual void resize(const sf::Vector2i&);
	
	virtual ~Game();
};

#endif // GAME_HPP_INCLUDED
