/*
 * statemanager.hpp - Kukatz 3D
 * Copyright (c) 2010, 2011 - TÖRÖK Attila (torokati44@gmail.com)
 *
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 *	 1. The origin of this software must not be misrepresented; you must not
 *			claim that you wrote the original software. If you use this software
 *			in a product, an acknowledgment in the product documentation would be
 *			appreciated but is not required.
 *
 *	 2. Altered source versions must be plainly marked as such, and must not be
 *			misrepresented as being the original software.
 *
 *	 3. This notice may not be removed or altered from any source
 *			distribution.
 */

#ifndef STATEMANAGER_HPP_INCLUDED
#define STATEMANAGER_HPP_INCLUDED

#include "opengl.hpp"

#include <SFML/Window.hpp>
#include <vector>
// should use some kind of linked list instead of vector?

#include "statebase.hpp"
#include "singleton.hpp"
#include "renderable.hpp"
#include "updatable.hpp"
#include "eventhandler.hpp"

class StateManager : public Singleton< StateManager >, public Updatable,
	public Renderable, public EventHandler
{
private:
	std::vector< StateBase* > state_stack;
	
public:
	StateManager();
	
	void resize_all(const sf::Vector2i&);
	
	StateBase* get_top_state();
	
	void push_state(StateBase*);
	
	void clear();
	bool is_empty();
	
	virtual void process_events(std::vector< sf::Event >&); // the uppermost state
	virtual void update(float); // the uppermost state
	virtual void render(); // the uppermost state
	
	virtual ~StateManager();
};

#endif //STATEMANAGER_HPP_INCLUDED
