/*
 * string.hpp - Kukatz 3D
 * Copyright (c) 2011 - TÖRÖK Attila (torokati44@gmail.com)
 *
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 *	 1. The origin of this software must not be misrepresented; you must not
 *			claim that you wrote the original software. If you use this software
 *			in a product, an acknowledgment in the product documentation would be
 *			appreciated but is not required.
 *
 *	 2. Altered source versions must be plainly marked as such, and must not be
 *			misrepresented as being the original software.
 *
 *	 3. This notice may not be removed or altered from any source
 *			distribution.
 */

#ifndef STRING_HPP_INCLUDED
#define STRING_HPP_INCLUDED

#include <string>

#include <SFML/System/Unicode.hpp>
#include <SFML/System/Vector2.hpp>
#include <SFML/Graphics/Color.hpp>

#include "renderable.hpp"
#include "font.hpp"

class String: public Renderable
{
private:
	const std::string font_id;
	
	sf::Unicode::UTF32String text;
	bool contains_color_reset;
	sf::Color color;
	
	GLint displaylist;
	
	void make_list();
	
protected:
	Font* font_ptr;
	sf::Vector2i bottom_left;
	sf::Vector2i top_right;
	
public:
	String(const sf::Unicode::Text&, const std::string&, float);
	String(const sf::Unicode::Text&, const std::string&);
	
	virtual void set_text(const sf::Unicode::Text&);
	sf::Unicode::Text get_text();
	
	void set_color(const sf::Color&);
	void set_color(sf::Uint8, sf::Uint8, sf::Uint8, sf::Uint8 = 255);
	const sf::Color& get_color();
	
	float get_width();
	float get_height();
	unsigned int num_lines();
	
	virtual void render();
	
	bool billboard;
	float size;
	
	virtual ~String();
};

#endif // STRING_HPP_INCLUDED
