/*
 * configmanager.cpp - Kukatz 3D
 * Copyright (c) 2011 - TÖRÖK Attila (torokati44@gmail.com)
 *
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 *   1. The origin of this software must not be misrepresented; you must not
 *      claim that you wrote the original software. If you use this software
 *      in a product, an acknowledgment in the product documentation would be
 *      appreciated but is not required.
 *
 *   2. Altered source versions must be plainly marked as such, and must not be
 *      misrepresented as being the original software.
 *
 *   3. This notice may not be removed or altered from any source
 *      distribution.
 */

#include "configmanager.hpp"

#include <iostream>
#include <fstream>
#include <cstring>
#include <sstream>

ConfigManager::ConfigManager()
{
	//  --------  DEFAULTS  --------
	
	values["colorfulness"] = std::string("1");
	values["blockyness"] = std::string("2");
	values["motionblur"] = std::string("3");
	values["particles"] = std::string("1");
	values["music"] = std::string("1");
	values["sound"] = std::string("1");
	
	//  --------  CONFIGFILE  --------
	
	std::ifstream file("config");
	
	if (file.is_open())
	{
		std::string line;
		while (!file.eof())
		{
			line = "";
			file >> line;
			
			if ((line.length()) > 0 && (line[0] != '#'))
			{
				std::string name(strtok((char*)line.c_str(), " ="));
				std::string value(strtok(0, " ="));
				
				if ((name.length() > 0) && (value.length() > 0))
				{
					values[name] = value;
				}
			}
		}
	}
}

void ConfigManager::save()
{
	std::ofstream file("config", std::ios_base::trunc);
	if (file.is_open())
	{
		std::map<std::string, std::string>::const_iterator end = values.end();
		for (std::map<std::string, std::string>::const_iterator i = values.begin(); i != end; ++i)
		{
			file << i->first << "=" << i->second << "\n";
		}
	}
	else
	{
		std::cerr << "ERROR! Couldn't open config file to save configuration.\n";
	}
}

bool ConfigManager::is_set(std::string k)
{
	return (values.count(k) != 0);
}

bool ConfigManager::get_bool(std::string k)
{
	std::string& v = values[k];
	return ((v == "1") || (v == "y") || (v == "Y") || (v == "yes") ||
		(v == "YES") || (v == "Yes") || (v == "on") || (v == "ON") || (v == "On"));
}

int ConfigManager::get_int(std::string k)
{
	ss.clear();
	ss.str("");
	
	ss.str(values[k]);
	
	int ret;
	
	ss >> ret;
	
	return ret;
}

float ConfigManager::get_float(std::string k)
{
	ss.clear();
	ss.str("");
	
	ss.str(values[k]);
	
	float ret;
	
	ss >> ret;
	
	return ret;
}

std::string ConfigManager::get_string(std::string k)
{
	return values[k];
}

void ConfigManager::set(std::string k, bool v)
{
	values[k] = std::string(v?"1":"0");
}

void ConfigManager::set(std::string k, int v)
{
	ss.clear();
	ss.str("");
	
	ss << v;
	
	values[k] = ss.str();
}

void ConfigManager::set(std::string k, float v)
{
	ss.clear();
	ss.str("");
	
	ss << v;
	
	values[k] = ss.str();
}

void ConfigManager::set(std::string k, std::string v)
{
	values[k] = v;
}

ConfigManager::~ConfigManager()
{
	save();
}
