/*
 * game.cpp - XD-Ball
 * Copyright (c) 2011, 2012 - TÖRÖK Attila (torokati44@gmail.com)
 *
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 *   1. The origin of this software must not be misrepresented; you must not
 *      claim that you wrote the original software. If you use this software
 *      in a product, an acknowledgment in the product documentation would be
 *      appreciated but is not required.
 *
 *	 2. Altered source versions must be plainly marked as such, and must not be
 *      misrepresented as being the original software.
 *
 *	 3. This notice may not be removed or altered from any source
 *      distribution.
 */

#include "game.hpp"

#include <iostream>
#include <cmath>
#include <cstdio>

#include <SFML/System/Randomizer.hpp>

#include "statemanager.hpp"
#include "resources.hpp"
#include "gameover.hpp"
#include "configmanager.hpp"

Game::Game(unsigned int w, unsigned int h):
	StateBase(w, h), score(0.0f), combo(0), time(0.0f)
{
	balls.push_back(new Ball(sf::Vector2f(400, 100), sf::Vector2f(20, 20), sf::Vector2f(0, 400)));
	paddle = new Paddle(sf::Vector2f(400, 50));
	
	music = new sf::Music();
	if (ConfigManager::instance().get_bool("music"))
	{
		if (sf::Randomizer::Random(0, 1) == 0)
		{
			music->OpenFromMemory((const char*)&(Resources::instance().tracks["RICK"]->front()), Resources::instance().tracks["RICK"]->size());
		}
		else
		{
			music->OpenFromMemory((const char*)&(Resources::instance().tracks["LOVE"]->front()), Resources::instance().tracks["LOVE"]->size());
		}
		music->SetVolume(75.0f);
		music->Play();
	}
	
	for (unsigned int i = 0; i < 20; ++i)
	{
		for (unsigned int j = 0; j < 16; ++j)
		{
			if (sf::Randomizer::Random(0, 4) == 0)
			{
				bricks.push_back(new Brick(sf::Vector2f(i * 40, 550 - (j + 1) * 20), sf::Vector2f(40, 20)));
				bricks.back()->color = sf::Color(sf::Randomizer::Random(32, 255), sf::Randomizer::Random(32, 255), sf::Randomizer::Random(64, 255), 255);
			}
		}
	}
	
	
	
	gui.size.x = w;
	gui.size.y = h;
	
	sprintf(buffer, "COMBO: %d", combo);
	
	combo_string = new GUIString(buffer, "EDITUNDO");
	gui.attach(combo_string, 0.08, GUIElement::SCALE_MODE_VERTICAL, 0.0f, 1.0f);
	
	
	sprintf(buffer, "SCORE: %d", score);
	
	score_string = new GUIString(buffer, "EDITUNDO");
	gui.attach(score_string, 0.08, GUIElement::SCALE_MODE_VERTICAL, 1.0f, 1.0f);
}

void Game::process_events(std::vector< sf::Event >& events)
{
	for (unsigned int i = 0; i < events.size(); ++i)
	{
		switch (events[i].Type)
		{
			case sf::Event::KeyPressed:
			{
				switch (events[i].Key.Code)
				{
					case sf::Key::Escape:
					{
						pop();
					}
					break;
					/*
					// HACK! HACK! HACK! HACK!
					
					case sf::Key::Space:
					{
						for (unsigned int j = 0; j < 50; ++j)
						{
							update(0.01f);
						}
					}
					break;
					
					// END OF HACK
					*/
					default:
					{
						
					}
					break;
				}
			}
			break;
			case sf::Event::MouseMoved:
			{
				paddle->position.x += (events[i].MouseMove.X - 400);
				
				if (paddle->position.x < (paddle->size.x / 2.0f))
				{
					paddle->position.x = (paddle->size.x / 2.0f);
				}
				else
				{
					if (paddle->position.x > (800 - (paddle->size.x / 2.0f)))
					{
						paddle->position.x = (800 - (paddle->size.x / 2.0f));
					}
				}
			}
			break;
			default:
			{
				
			}
			break;
		}
	}
	
	events.clear();
}

void Game::render()
{
	background.render();
	
	for (unsigned int i = 0; i < bricks.size(); ++i)
	{
		bricks[i]->render();
	}
	
	for (unsigned int i = 0; i < balls.size(); ++i)
	{
		balls[i]->render();
	}
	
	paddle->render();
	
	gui.render();
	
	particle_system.render();
}

void Game::update(float dt)
{
	time += dt;
	sf::Color color(
		std::min(std::max(       (float) fabs(fmod(time * std::max(combo, 1u), 6.0f) - 3.0f) - 1.0f,  0.0f), 1.0f) * 255.0f,
		std::min(std::max(1.0f - (float)(fabs(fmod(time * std::max(combo, 1u), 6.0f) - 2.0f) - 1.0f), 0.0f), 1.0f) * 255.0f,
		std::min(std::max(1.0f - (float)(fabs(fmod(time * std::max(combo, 1u), 6.0f) - 4.0f) - 1.0f), 0.0f), 1.0f) * 255.0f);
	
	sf::Color score_color = score_string->get_color();
	if (score_color.r < 255)
	{
		score_color.r += 1;
	}
	if (score_color.g < 255)
	{
		score_color.g += 1;
	}
	if (score_color.b < 255)
	{
		score_color.b += 1;
	}
	score_string->set_color(score_color);
	
	if (combo > 1)
	{
		combo_string->set_color(color);
	}
	else
	{
		if (combo == 0)
		{
			combo_string->set_color(255, 255, 255, 128);
		}
		else
		{
			combo_string->set_color(255, 255, 255, 255);
		}
	}
	
	for (unsigned int i = 0; i < balls.size(); ++i)
	{
		balls[i]->update(dt);
		
		sf::Vector2f ball_bl = balls[i]->position - balls[i]->origin;
		sf::Vector2f ball_tr = ball_bl + balls[i]->size;
		const sf::Vector2f& ball_pos = balls[i]->position;
		sf::Vector2f& ball_speed = balls[i]->speed;
		sf::Vector2f old_ball_speed = balls[i]->speed;
		
		for (unsigned int j = 0; j < bricks.size(); ++j)
		{
			if (balls[i]->intersects(*bricks[j]))
			{
				Ball::Collision coll = balls[i]->collide(bricks[j]);
				if (coll.happened)
				{
					if (ConfigManager::instance().get_bool("particles"))
					{
						particle_system.sparkle(coll.position, coll.speed);
						particle_system.sparkle(bricks[j]->position + bricks[j]->size / 2.0f, (old_ball_speed - ball_speed) / 10.0f, 20, 2, bricks[j]->color);
					}
					
					play_sound("RICOCHET0", coll.position.x);
					
					delete bricks[j];
					bricks[j] = bricks.back();
					bricks.pop_back();
					
					combo += 1;
					
					score += combo;
					
					sprintf(buffer, "COMBO: %d", combo);
					combo_string->set_text(buffer);
					
					sprintf(buffer, "SCORE: %d", score);
					score_string->set_text(buffer);
					score_string->set_color(255 - color.r, 255 - color.g, 255 - color.b);
					
					if (bricks.empty())
					{
						pop();
						StateManager::instance().push_state(new GameOver(800, 600, true, score));
					}
					
					break;
				}
			}
		}
		
		if (balls[i]->intersects(*paddle))
		{
			Ball::Collision coll = balls[i]->collide(paddle);
			if (coll.happened)
			{
				play_sound("PADDLE", ball_pos.x);
				combo = 0;
				sprintf(buffer, "COMBO: %d", combo);
				combo_string->set_text(buffer);
			}
		}
		
		if (ball_bl.x <= 0)
		{
			play_sound("KNOCK", 0);
			ball_speed.x = fabs(ball_speed.x);
		}
		
		if (ball_bl.y <= 0)
		{
			//ball_speed.y = fabs(ball_speed.y);
			pop();
			StateManager::instance().push_state(new GameOver(800, 600, false, score));
		}
		
		if (ball_tr.x >= 800)
		{
			play_sound("KNOCK", 800);
			ball_speed.x = -fabs(ball_speed.x);
		}
		
		if (ball_tr.y >= 600)
		{
			play_sound("KNOCK", ball_pos.x);
			ball_speed.y = -fabs(ball_speed.y);
		}
	}
	
	background.update(dt);
	
	particle_system.update(dt);
	
	for (unsigned int i = 0; i < sounds.size(); ++i)
	{
		if (sounds[i]->GetStatus() != sf::Sound::Playing)
		{
			delete sounds[i];
			sounds[i] = sounds.back();
			sounds.pop_back();
		}
	}
	
	if (ConfigManager::instance().get_bool("music"))
	{
		if (music->GetStatus() != sf::Music::Playing)
		{
			if (sf::Randomizer::Random(0, 1) == 0)
			{
				music->OpenFromMemory((const char*)&(Resources::instance().tracks["RICK"]->front()), Resources::instance().tracks["RICK"]->size());
			}
			else
			{
				music->OpenFromMemory((const char*)&(Resources::instance().tracks["LOVE"]->front()), Resources::instance().tracks["LOVE"]->size());
			}
			music->SetVolume(75.0f);
			music->Play();
		}
	}
}

void Game::resize(const sf::Vector2i&)
{
	
}

void Game::play_sound(const std::string& str, float x)
{
	if (ConfigManager::instance().get_bool("sound"))
	{
		sounds.push_back(new sf::Sound(*Resources::instance().sounds[str]));
		sounds.back()->SetPosition(sf::Vector3f((x - 400.0f) / 400.0f, 0, 0.0f));
		sounds.back()->Play();
	}
}

Game::~Game()
{
	for (unsigned int i = 0; i < balls.size(); ++i)
	{
		delete balls[i];
	}
	balls.clear();
	
	for (unsigned int i = 0; i < bricks.size(); ++i)
	{
		delete bricks[i];
	}
	bricks.clear();
	
	for (unsigned int i = 0; i < sounds.size(); ++i)
	{
		delete sounds[i];
	}
	sounds.clear();
	
	delete paddle;
	delete music;
}
