/*
 * guielementcontainer.cpp - Kukatz 3D
 * Copyright (c) 2011 - TÖRÖK Attila (torokati44@gmail.com)
 *
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 *	 1. The origin of this software must not be misrepresented; you must not
 *			claim that you wrote the original software. If you use this software
 *			in a product, an acknowledgment in the product documentation would be
 *			appreciated but is not required.
 *
 *	 2. Altered source versions must be plainly marked as such, and must not be
 *			misrepresented as being the original software.
 *
 *	 3. This notice may not be removed or altered from any source
 *			distribution.
 */

#include "guielementcontainer.hpp"

#include <cmath>
#include <iostream>

#include "misc.hpp"
#include "string.hpp"
#include "guielement.hpp"

GUIElementContainer::GUIElementContainer(float ar)
{
	aspect_ratio = ar;
}

void GUIElementContainer::attach(GUIElement* e)
{
	children.push_back(e);
}

void GUIElementContainer::attach(GUIElement* e, float s, ScaleMode sm,
	float ax, float ay)
{
	e->scale = s;
	e->scale_mode = sm;
	e->align.x = ax;
	e->align.y = ay;
	
	children.push_back(e);
}

void GUIElementContainer::render()
{
	for (unsigned int i = 0; i < children.size(); ++i)
	{
		sf::Vector2f child_size;
		
		switch (children[i]->scale_mode)
		{
			case SCALE_MODE_HORIZONTAL:
			{
				child_size.x = (float)size.x * children[i]->scale;
				child_size.y = child_size.x / children[i]->aspect_ratio;
			}
			break;
			case SCALE_MODE_VERTICAL:
			{
				child_size.y = (float)size.y * children[i]->scale;
				child_size.x = child_size.y * children[i]->aspect_ratio;
			}
			break;
			case SCALE_MODE_SMALLER:
			{
				if (((float)size.x * children[i]->scale) // horizontal
					< ((float)size.y * children[i]->scale * children[i]->aspect_ratio))
				{
					child_size.x = (float)size.x * children[i]->scale;
					child_size.y = child_size.x / children[i]->aspect_ratio;
				}
				else // vertical
				{
					child_size.y = (float)size.y * children[i]->scale;
					child_size.x = child_size.y * children[i]->aspect_ratio;
				}
			}
			break;
			case SCALE_MODE_BIGGER:
			{
				if (((float)size.x * children[i]->scale) // horizontal
					> ((float)size.y * children[i]->scale * children[i]->aspect_ratio))
				{
					child_size.x = (float)size.x * children[i]->scale;
					child_size.y = child_size.x / children[i]->aspect_ratio;
				}
				else // vertical
				{
					child_size.y = (float)size.y * children[i]->scale;
					child_size.x = child_size.y * children[i]->aspect_ratio;
				}
			}
			break;
		}
		
		children[i]->size = ftoi2(child_size);
		
		glPushMatrix();
		
		glTranslatef(((float)size.x - child_size.x) * children[i]->align.x,
			((float)size.y - child_size.y) * children[i]->align.y, 0);
		
		children[i]->render();
		
		glPopMatrix();
	}
}

GUIElementContainer::~GUIElementContainer()
{
	for (unsigned int i = 0; i < children.size(); ++i)
	{
		delete children[i];
	}
}
