/*
 * mainmenu.cpp - XD-Ball
 * Copyright (c) 2010, 2011, 2012 - TÖRÖK Attila (torokati44@gmail.com)
 *
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 *	 1. The origin of this software must not be misrepresented; you must not
 *			claim that you wrote the original software. If you use this software
 *			in a product, an acknowledgment in the product documentation would be
 *			appreciated but is not required.
 *
 *	 2. Altered source versions must be plainly marked as such, and must not be
 *			misrepresented as being the original software.
 *
 *	 3. This notice may not be removed or altered from any source
 *			distribution.
 */

#include "mainmenu.hpp"

#include "game.hpp"
#include "statemanager.hpp"
#include "optionsmenu.hpp"
#include "misc.hpp"
#include "guistring.hpp"
#include "resources.hpp"
#include "configmanager.hpp"

MainMenu::MainMenu(unsigned int w, unsigned int h): StateBase(w, h),
	current_item(0), time(0.0f)
{
	resize(sf::Vector2i(w, h));
	
	items.push_back(new GUIString("NEW GAME", "EDITUNDO"));
	items.push_back(new GUIString("OPTIONS", "EDITUNDO"));
	items.push_back(new GUIString("QUIT", "EDITUNDO"));
	
	title = new GUIString("XD-BALL", "EDITUNDO");
	
	GUIElementContainer* square = new GUIElementContainer(4.0f / 3.0f);
	gui.attach(square, 1.0f, GUIElement::SCALE_MODE_SMALLER, 0.5f, 0.5f);
	
	for (unsigned int i = 0; i < items.size(); ++i)
	{
		square->attach(items[i], 0.75f / (float)(items.size() + 2),
			GUIElement::SCALE_MODE_VERTICAL, 0.5,
			1.1f - ((float)i + 2.5f) / (float)(items.size() + 2));
	}
	
	square->attach(title, 1.0f / (float)(items.size() + 1),
		GUIElement::SCALE_MODE_VERTICAL, 0.5f,
		1.1f - (0.5f / (float)(items.size() + 1)));
	
	square->attach(new GUIString(L"MADE BY ATTILA TÖRÖK IN 2012", "EDITUNDO"), 0.05f, GUIElement::SCALE_MODE_VERTICAL,        0.5f, 0.1f);
	square->attach(new GUIString(L"THANKS TO BJARNE STROUSTRUP FOR C++", "EDITUNDO"), 0.05f, GUIElement::SCALE_MODE_VERTICAL, 0.5f, 0.05f);
	square->attach(new GUIString(L"AND TO LAURENT GOMILA FOR SFML", "EDITUNDO"), 0.05f, GUIElement::SCALE_MODE_VERTICAL,      0.5f, 0.0f);
	
	music.OpenFromMemory((const char*)&(Resources::instance().tracks["NYAN"]->front()), Resources::instance().tracks["NYAN"]->size());
	music.SetVolume(75.0f);
	music.SetLoop(true);
}

void MainMenu::resize(const sf::Vector2i& s)
{
	gui.size = s;
	size = s;
}

void MainMenu::process_events(std::vector< sf::Event >& events)
{
	for (size_t i = 0; i < events.size(); ++i)
	{
		if ((events[i].Type == sf::Event::KeyPressed))
		{
			switch(events[i].Key.Code)
			{
				case sf::Key::Escape:
				{
					pop();
				}
				break;
				case sf::Key::Return:
				{
					switch (current_item)
					{
						case 0:
						{
							music.Stop();
							StateManager::instance().push_state(new Game(size.x, size.y));
						}
						break;
						case 1:
						{
							StateManager::instance().push_state(new OptionsMenu(size.x, size.y));
						}
						break;
						case 2:
						{
							pop();
						}
						break;
					}
				}
				break;
				case sf::Key::Up:
				{
					items[current_item]->set_color(sf::Color::White);
					if (current_item == 0)
					{
						current_item = items.size() - 1;
					}
					else
					{
						--current_item;
					}
					items[current_item]->set_color(sf::Color::Red);
				}
				break;
				case sf::Key::Down:
				{
					items[current_item]->set_color(sf::Color::White);
					if (current_item == items.size() - 1)
					{
						current_item = 0;
					}
					else
					{
						++current_item;
					}
					items[current_item]->set_color(sf::Color::Red);
				}
				break;
				default:
				{
					
				}
				break;
			}
		}
	}
}

void MainMenu::update(float dt)
{
	time += dt;
	background.update(dt);
	
	sf::Color color(
		std::min(std::max(       (float) fabs(fmod(time * 5.0f, 6.0f) - 3.0f) - 1.0f,  0.0f), 1.0f) * 255.0f,
		std::min(std::max(1.0f - (float)(fabs(fmod(time * 5.0f, 6.0f) - 2.0f) - 1.0f), 0.0f), 1.0f) * 255.0f,
		std::min(std::max(1.0f - (float)(fabs(fmod(time * 5.0f, 6.0f) - 4.0f) - 1.0f), 0.0f), 1.0f) * 255.0f);
	
	title->set_color(color);
	items[current_item]->set_color(255 - color.r, 255 - color.g, 255 - color.b);
	
	if (ConfigManager::instance().get_bool("music"))
	{
		if (music.GetStatus() != sf::Music::Playing)
		{
			music.SetVolume(75.0f);
			music.Play();
		}
	}
	else
	{
		music.Stop();
	}
}

void MainMenu::render()
{
	background.render();
	gui.render();
}

MainMenu::~MainMenu()
{
	// no need to delete the items, the gui will delete them
}
