/*
 * optionsmenu.cpp - Kukatz 3D
 * Copyright (c) 2011 - TÖRÖK Attila (torokati44@gmail.com)
 *
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 *   1. The origin of this software must not be misrepresented; you must not
 *      claim that you wrote the original software. If you use this software
 *      in a product, an acknowledgment in the product documentation would be
 *      appreciated but is not required.
 *
 *   2. Altered source versions must be plainly marked as such, and must not be
 *      misrepresented as being the original software.
 *
 *   3. This notice may not be removed or altered from any source
 *      distribution.
 */

#include "optionsmenu.hpp"

#include <sstream>
#include <cmath>

#include "configmanager.hpp"
#include "statemanager.hpp"
#include "guistring.hpp"
#include "selector.hpp"

OptionsMenu::OptionsMenu(unsigned int w, unsigned int h): StateBase(w, h),
	current_item(0), time(0.0f)
{
	gui.size.x = w;
	gui.size.y = h;
	
	GUIElementContainer* square = new GUIElementContainer(4.0f / 3.0f);
	gui.attach(square, 1.0f, GUIElement::SCALE_MODE_SMALLER, 0.5f, 0.5f);
	
	title = new GUIString("OPTIONS", "EDITUNDO");
	square->attach(title, 0.2, GUIString::SCALE_MODE_VERTICAL, 0.5, 1);
	
	
	std::vector< sf::Unicode::Text > opt;
	
	opt.clear();
	opt.push_back("0");
	opt.push_back("1");
	opt.push_back("2");
	opt.push_back("3");
	opt.push_back("4");
	
	Selector* sel = new Selector("COLORFULNESS:", opt, ConfigManager::instance().get_int("colorfulness"), false, 16.0f, 0.6f);
	sel->set_color(255, 0, 0);
	items.push_back(sel);
	square->attach(sel, 0.8f, GUIElement::SCALE_MODE_HORIZONTAL, 0.5, 0.8);
	
	opt.push_back("5");
	
	sel = new Selector("BLOCKYNESS:", opt, ConfigManager::instance().get_int("blockyness"), false, 16.0f, 0.6f);
	items.push_back(sel);
	square->attach(sel, 0.8f, GUIElement::SCALE_MODE_HORIZONTAL, 0.5, 0.7);
	
	opt.push_back("6");
	opt.push_back("7");
	opt.push_back("8");
	opt.push_back("9");
	
	sel = new Selector("MOTION BLUR:", opt, ConfigManager::instance().get_int("motionblur"), false, 16.0f, 0.6f);
	items.push_back(sel);
	square->attach(sel, 0.8f, GUIElement::SCALE_MODE_HORIZONTAL, 0.5, 0.6);
	
	opt.clear();
	opt.push_back("OFF");
	opt.push_back("ON");
	
	sel = new Selector("PARTICLES:", opt, ConfigManager::instance().get_bool("particles")?1:0, true, 16.0f, 0.6f);
	items.push_back(sel);
	square->attach(sel, 0.8f, GUIElement::SCALE_MODE_HORIZONTAL, 0.5, 0.5);
	
	sel = new Selector("MUSIC:", opt, ConfigManager::instance().get_bool("music")?1:0, true, 16.0f, 0.6f);
	items.push_back(sel);
	square->attach(sel, 0.8f, GUIElement::SCALE_MODE_HORIZONTAL, 0.5, 0.4);
	
	sel = new Selector("SOUND:", opt, ConfigManager::instance().get_bool("sound")?1:0, true, 16.0f, 0.6f);
	items.push_back(sel);
	square->attach(sel, 0.8f, GUIElement::SCALE_MODE_HORIZONTAL, 0.5, 0.3);
}

void OptionsMenu::resize(const sf::Vector2i& s)
{
	gui.size = s;
	size = s;
}

void OptionsMenu::process_events(std::vector< sf::Event >& events)
{
	for (unsigned int i = 0; i < events.size(); ++i)
	{
		if (events[i].Type == sf::Event::KeyPressed)
		{
			switch(events[i].Key.Code)
			{
				case sf::Key::Escape:
				{
					ConfigManager::instance().set("colorfulness", (int)items[0]->get_option());
					ConfigManager::instance().set("blockyness", (int)items[1]->get_option());
					ConfigManager::instance().set("motionblur", (int)items[2]->get_option());
					ConfigManager::instance().set("particles", items[3]->get_option() == 1);
					ConfigManager::instance().set("music", items[4]->get_option() == 1);
					ConfigManager::instance().set("sound", items[5]->get_option() == 1);
					ConfigManager::instance().save();
					pop();
				}
				break;
				case sf::Key::Down:
				{
					items[current_item]->set_color(255, 255, 255);
					if (current_item == items.size()-1)
					{
						current_item = 0;
					}
					else
					{
						++current_item;
					}
				}
				break;
				case sf::Key::Up:
				{
					items[current_item]->set_color(255, 255, 255);
					if (current_item == 0)
					{
						current_item = items.size() - 1;
					}
					else
					{
						--current_item;
					}
				}
				break;
				case sf::Key::Left:
				{
					items[current_item]->prev();
				}
				break;
				case sf::Key::Right:
				{
					items[current_item]->next();
				}
				break;
				default:
				{
					
				}
				break;
			}
		}
	}
}

void OptionsMenu::update(float dt)
{
	time += dt;
	background.update(dt);
	
	sf::Color color(
		std::min(std::max(       (float) fabs(fmod(time * 5.0f, 6.0f) - 3.0f) - 1.0f,  0.0f), 1.0f) * 255.0f,
		std::min(std::max(1.0f - (float)(fabs(fmod(time * 5.0f, 6.0f) - 2.0f) - 1.0f), 0.0f), 1.0f) * 255.0f,
		std::min(std::max(1.0f - (float)(fabs(fmod(time * 5.0f, 6.0f) - 4.0f) - 1.0f), 0.0f), 1.0f) * 255.0f);
	
	title->set_color(color);
	items[current_item]->set_color(255 - color.r, 255 - color.g, 255 - color.b);
}

void OptionsMenu::render()
{
	background.render();
	gui.render();
}

OptionsMenu::~OptionsMenu()
{
	// no need to delete the items, the gui will delete it
}
