/*
 * particlesystem.cpp - XD-Ball
 * Copyright (c) 2011 - TÖRÖK Attila (torokati44@gmail.com)
 *
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 *   1. The origin of this software must not be misrepresented; you must not
 *      claim that you wrote the original software. If you use this software
 *      in a product, an acknowledgment in the product documentation would be
 *      appreciated but is not required.
 *
 *	 2. Altered source versions must be plainly marked as such, and must not be
 *      misrepresented as being the original software.
 *
 *	 3. This notice may not be removed or altered from any source
 *      distribution.
 */

#include "particlesystem.hpp"

#include <cmath>

#include <SFML/System/Randomizer.hpp>

ParticleSystem::ParticleSystem()
{
	
}

void ParticleSystem::update(float dt)
{
	for (unsigned int i = 0; i < particles.size(); ++i)
	{
		particles[i]->update(dt);
		
		if (particles[i]->is_dead())
		{
			delete particles[i];
			particles[i] = particles.back();
			particles.pop_back();
		}
	}
}

void ParticleSystem::render()
{
	for (unsigned int i = 0; i < particles.size(); ++i)
	{
		particles[i]->render();
	}
}

void ParticleSystem::sparkle(const sf::Vector2f& p, const sf::Vector2f& s, const sf::Color& c)
{
	float len = sqrtf(s.x*s.x + s.y*s.y);
	sparkle(p, s, 10 + len / 100.0f, len / 200.0f, c);
}

void ParticleSystem::sparkle(const sf::Vector2f& p, const sf::Vector2f& s, int num, float dur, const sf::Color& color)
{
	for (unsigned int i = 0; i < num; ++i)
	{
		sf::Vector2f speed = s;
		speed.x += sf::Randomizer::Random(-100.0f, 100.0f);
		speed.y += sf::Randomizer::Random(-100.0f, 100.0f);
		particles.push_back(new Particle(p, speed, dur + sf::Randomizer::Random(-0.5f, 0.5f), color));
	}
}

ParticleSystem::~ParticleSystem()
{
	for (unsigned int i = 0; i < particles.size(); ++i)
	{
		delete particles[i];
	}
	particles.clear();
}
