/*
 * sprite.cpp - XD-Ball
 * Copyright (c) 2011 - TÖRÖK Attila (torokati44@gmail.com)
 *
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 *   1. The origin of this software must not be misrepresented; you must not
 *      claim that you wrote the original software. If you use this software
 *      in a product, an acknowledgment in the product documentation would be
 *      appreciated but is not required.
 *
 *	 2. Altered source versions must be plainly marked as such, and must not be
 *      misrepresented as being the original software.
 *
 *	 3. This notice may not be removed or altered from any source
 *      distribution.
 */

#include "sprite.hpp"

#include <iostream>

#include "resources.hpp"

GLuint Sprite::display_list = 0;
unsigned int Sprite::num_sprites = 0;

Sprite::Sprite(const std::string& t, const sf::Vector2f& p,
		const sf::Vector2f& s, const sf::Vector2f& o, float a, const sf::Color& c):
	texture_ptr(Resources::instance().textures[t]), position(p), size(s),
	origin(o), angle(a), color(c)
{
	++num_sprites;
	make_list();
}

Sprite::Sprite(const std::string& t, const sf::Vector2f& p):
	texture_ptr(Resources::instance().textures[t]), position(p),
	size(texture_ptr->get_width(), texture_ptr->get_height()),
	origin(size.x / 2.0f, size.y / 2.0f), angle(0), color(255, 255, 255, 255)
{
	++num_sprites;
	make_list();
}

void Sprite::make_list()
{
	if (display_list == 0)
	{
		display_list = glGenLists(1);
		
		glNewList(display_list, GL_COMPILE);
		glBegin(GL_QUADS);
		
		glTexCoord2f(0, 0);
		glVertex2f(0, 0);
		
		glTexCoord2f(1, 0);
		glVertex2f(1, 0);
		
		glTexCoord2f(1, 1);
		glVertex2f(1, 1);
		
		glTexCoord2f(0, 1);
		glVertex2f(0, 1);
		
		glEnd();
		glEndList();
	}
}

void Sprite::render()
{
	texture_ptr->bind();
	glColor4ub(color.r, color.g, color.b, color.a);
	glPushMatrix();
	
	glTranslatef(position.x, position.y, 0.0f);
	glRotatef(angle, 0.0f, 0.0f, 1.0f);
	glScalef(size.x, size.y, 0.0f);
	glTranslatef(-origin.x / size.x, -origin.y / size.y, 0.0f);
	
	glCallList(display_list);
	glPopMatrix();
}

bool Sprite::intersects(const Sprite& other)
{
	sf::Vector2f bl = position - origin;
	sf::Vector2f tr = bl + size;
	
	sf::Vector2f other_bl = other.position - other.origin;
	sf::Vector2f other_tr = other_bl + other.size;
	
	return !((tr.x < other_bl.x) ||
					(bl.x > other_tr.x) ||
					(tr.y < other_bl.y) ||
					(bl.y > other_tr.y));;
}

Sprite::~Sprite()
{
	--num_sprites;
	if (num_sprites == 0)
	{
		glDeleteLists(display_list, 1);
		display_list = 0;
	}
}
