/*
 * texture.cpp - Kukatz 3D
 * Copyright (c) 2011 - TÖRÖK Attila (torokati44@gmail.com)
 *
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 *	 1. The origin of this software must not be misrepresented; you must not
 *			claim that you wrote the original software. If you use this software
 *			in a product, an acknowledgment in the product documentation would be
 *			appreciated but is not required.
 *
 *	 2. Altered source versions must be plainly marked as such, and must not be
 *			misrepresented as being the original software.
 *
 *	 3. This notice may not be removed or altered from any source
 *			distribution.
 */

#include "texture.hpp"

#include <iostream>
#include <vector>
#include <string>
#include <fstream>

#include "opengl.hpp"

// Kind of ugly and hacky, but I didn't want to change the original file.
// Although that unused known_type variable could have been removed safely.
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wunused-but-set-variable"
#include "picopng.hpp"
#pragma GCC diagnostic pop

GLuint Texture::bound_texture = 0;

Texture::Texture(const std::string& filename)
{
	std::ifstream file(filename.c_str(), std::ios::binary);
	
	if (!(file.is_open() && file.good()))
	{
		std::cerr << "ERROR! Couldn't load texture \"" << filename << "\".\n";
		return;
	}
	
	file.seekg(0, std::ios::end);
	sf::Uint32 data_size = file.tellg();
	file.seekg(0, std::ios::beg);
	
	sf::Uint8 data[data_size];
	
	file.read((char*)data, data_size);
	
	if (!file.good())
	{
		std::cerr << "ERROR! Couldn't load texture \"" << filename << "\".\n";
		return;
	}
	
	std::vector<unsigned char> pixels;
	
	if (decodePNG(pixels, width, height, data, data_size, true))
	{
		std::cerr << "ERROR! Something went wrong with the decoding of texture \"" << filename << "\".\n";
		return;
	}
	
	glGenTextures(1, &gl_id);
	glBindTexture(GL_TEXTURE_2D, gl_id);
	
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, &pixels[0]);
	glGenerateMipmap(GL_TEXTURE_2D);
}

void Texture::bind()
{
	if (bound_texture != gl_id)
	{
		glBindTexture(GL_TEXTURE_2D, gl_id);
		bound_texture = gl_id;
	}
}

void Texture::unbind()
{
	glBindTexture(GL_TEXTURE_2D, 0);
	bound_texture = 0;
}

unsigned long Texture::get_width() const
{
	return width;
}

unsigned long Texture::get_height() const
{
	return height;
}

Texture::~Texture()
{
	glDeleteTextures(1, &gl_id);
}
